<?php
/*
 * @title Converter for RAKUTEN Recipe Data
 * @author Keisuke Doman <kdoman@sist.chukyo-u.ac.jp>
 * @update Aug. 11, 2014
 */


// PHP settings
error_reporting( E_ALL );
set_time_limit( 0 );
ini_set( 'memory_limit', '-1' );

// constants
define( 'MAX_MATERIAL_NUM', 291993 );
define( 'MAX_RECIPE_NUM', 442504 );
define( 'MAX_TSUKUREPO_NUM', 244176 );
$database_recipe = 'rakuten_recipe.db';
$input_files = array( 'material' => 'recipe_material_20120705.txt',
					  'step' => 'recipe_process_20120705.txt',
					  'recipe' => 'recipe_all_20120705.txt',
					  'tsukurepo' => 'recipe_tsukurepo_20120705.txt' );
$temp_dir = './sorted';
$output_dir = './dat';
$output_files = array( 'cost' => 'costs.dat',
					   'moment' => 'moments.dat',
					   'time' => 'times.dat',
					   'material' => 'materials.dat',
					   'recipe' => 'recipes.dat',
					   'tsukurepo' => 'tsukurepos.dat' );
$cost_records = array( '100円以下', '300円前後', '500円前後', '1000円前後', '2000円前後', '3000円前後', '5000円前後', '10000円以上' );
$moment_records = array( '普段の料理', 'おもてなし', 'お祝い', 'おつまみ', 'おでかけ', '贈り物', '妊婦さん', '離乳食' );
$time_records = array( '5分以内', '約10分', '約15分', '約30分', '約1時間', '1時間以上' );

// function definision
function force_make_dir( $dirname )
{
	if( is_file( $dirname ) )
	{
		unlink( $dirname );
	}
	if( !is_dir( $dirname ) )
	{
		mkdir( $dirname );
	}
}

function sort_data( $in_filename, $out_filename, $key_index = 0, $separator = "\t" )
{
	echo 'sort: "'.$in_filename.'" -> "'.$out_filename.'"'."\n";
	$data = array( );

	$lines = file( $in_filename );
    foreach( $lines as $line )
	{
    	$parts = explode( $separator, $line, ( $key_index + 1 ) + 1 ); // index + 2 is enough for running the following steps
		for( $i = 1; isset( $data[ $parts[ $key_index ].'_'.sprintf( '%02d', $i ) ] ); ++$i )
		{
			;
		}
		$data[ $parts[ $key_index ].'_'.sprintf( '%02d', $i ) ] = $line;
	}
	unset( $lines );
	
	ksort( $data );
	
	file_put_contents( $out_filename, $data );
}

function output_data( $filename, $data, $echo_prefix = "", $echo_interval = 10000, $separator = "\t", $append = false )
{
	$fp = fopen( $filename, 'w' );

	if( !empty( $echo_prefix ) ) echo $echo_prefix.": 0 (0%)\r";
	for( $i = 0, $data_count = count( $data ); $i < $data_count; ++$i )
	{
		fwrite( $fp, ( $i + 1 ).$separator.str_replace( '"', '\"', implode( $separator, is_array( $data[ $i ] ) ? $data[ $i ] : array( $data[ $i ] ) ) )."\n" );
		if( !empty( $echo_prefix ) && ( $i + 1 ) % $echo_interval == 0 ) echo $echo_prefix.': '.( $i + 1 ).' ('.round( 100 * ( $i + 1 ) / $data_count )."%)\r";
	}
	if( !empty( $echo_prefix ) ) echo $echo_prefix.': '.$i.' ('.round( 100 * $i / $data_count )."%)\n";

	fclose( $fp );
}

function make_material_title_id_map( $filename, &$material_title_id_map )
{
	$material_title_id_map = array( );
	$fp_material = fopen( $filename, 'r' );

	echo "materials: 0 (0%)\r";
	for( $i = 0, $id = 1; $line = fgets( $fp_material ); ++$i )
	{
		$material_row = explode( "\t", $line );
		
		if( !isset( $material_title_id_map[ $material_row[ 1 ] ] ) )
		{
			$material_title_id_map += array( $material_row[ 1 ] => $id );
			if( $id % 50000 == 0 ) echo 'materials: '.$id.' ('.round( 100 * $id / MAX_MATERIAL_NUM )."%)\r";
			++$id;
		}
	}
	echo 'materials: '.$id.' ('.round( 100 * $id / MAX_MATERIAL_NUM )."%)\n";

	fclose( $fp_material );
}

function output_recipes( $in_recipe_filename, $in_material_filename, $in_step_filename, $out_filename, &$material_title_id_map )
{
	$fp_in_all = fopen( $in_recipe_filename, 'r' );
	$fp_in_material = fopen( $in_material_filename, 'r' );
	$fp_in_step = fopen( $in_step_filename, 'r' );
	$fp_out_recipe = fopen( $out_filename, 'w' );
	$material_row = $step_row = array( 0 );

	echo "recipes: 0 (0%)\r";
	for( $i = 0; $line_all = fgets( $fp_in_all ); ++$i )
	{
		$recipe_parts = explode( "\t", str_replace( "\0", '', trim( $line_all ) ), 20 );
		if( empty( $recipe_parts ) )
		{
			break;
		}
		
		$material_parts = array_pad( array( ), 2 * 36, '' ); // materialは72個（idとvolumeで半分ずつ）
		while( intval( $material_row[ 0 ] ) < intval( $recipe_parts[ 0 ] ) && $material_row = explode( "\t", str_replace( "\0", '', fgets( $fp_in_material ) ), 3 ) ) ;
		for( $r = 0; $recipe_parts[ 0 ] === $material_row[ 0 ]; ++$r )
		{
			$material_parts[ 2 * $r ] = $material_title_id_map[ $material_row[ 1 ] ];
			$material_parts[ 2 * $r + 1 ] = trim( $material_row[ 2 ] );
			$material_row = explode( "\t", str_replace( "\0", '', fgets( $fp_in_material ) ), 3 );
		}

		$step_parts = array_pad( array( ), 50, '' ); // stepは50個
		while( intval( $step_row[ 0 ] ) < intval( $recipe_parts[ 0 ] ) && $step_row = explode( "\t", str_replace( "\0", '', fgets( $fp_in_step ) ), 3 ) ) ;
		for( $r = 0; $recipe_parts[ 0 ] === $step_row[ 0 ]; ++$r )
		{
			$step_parts[ $r ] = trim( $step_row[ 2 ] );
			$step_row = explode( "\t", str_replace( "\0", '', fgets( $fp_in_step ) ), 3 );
		}

		fwrite( $fp_out_recipe, str_replace( '"', '\"', implode( "\t", array_merge( $recipe_parts, $material_parts, $step_parts ) ) )."\n" );

		if( $i % 4000 == 0 ) echo 'recipes: '.$i.' ('.round( 100 * $i / MAX_RECIPE_NUM )."%)\r";
	}
	echo 'recipes: '.$i.' ('.round( 100 * $i / MAX_RECIPE_NUM )."%)\n";

	fclose( $fp_out_recipe );
	fclose( $fp_in_step );
	fclose( $fp_in_material );
	fclose( $fp_in_all );
}


///////////////////////////
// START
///////////////////////////
$time_start = time( true );

// sort input files
force_make_dir( $temp_dir );
sort_data( $input_files[ 'material' ], $temp_dir.'/'.$input_files[ 'material' ], 0 );
sort_data( $input_files[ 'step' ], $temp_dir.'/'.$input_files[ 'step' ], 0 );
sort_data( $input_files[ 'recipe' ], $temp_dir.'/'.$input_files[ 'recipe' ], 0 );

// output costs & moments & times & tsukurepos
force_make_dir( $output_dir );
output_data( $output_dir.'/'.$output_files[ 'cost' ], $cost_records, 'costs' );
output_data( $output_dir.'/'.$output_files[ 'moment' ], $moment_records, 'moments' );
output_data( $output_dir.'/'.$output_files[ 'time' ], $time_records, 'times' );
output_data( $output_dir.'/'.$output_files[ 'tsukurepo' ], explode( "\n", str_replace( "\r", '', trim( file_get_contents( $input_files[ 'tsukurepo' ] ) ) ) ), 'tsukurepos' );

// output materials
make_material_title_id_map( $temp_dir.'/'.$input_files[ 'material' ], $material_title_id_map );
output_data( $output_dir.'/'.$output_files[ 'material' ], array_keys( $material_title_id_map ) );

// output recipes
output_recipes( $temp_dir.'/'.$input_files[ 'recipe' ], $temp_dir.'/'.$input_files[ 'material' ], $temp_dir.'/'.$input_files[ 'step' ], $output_dir.'/'.$output_files[ 'recipe' ], $material_title_id_map );

///////////////////////////
// END
///////////////////////////
$timelimit = time( true ) - $time_start;
echo 'total '.$timelimit." [sec]\n";

?>
